@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -18,18 +18,35 @@
 #include "item.h"
 #include "case.h"
 #include <mobius/core/category_manager.h>
+#include <mobius/core/mediator.h>
 #include <mobius/exception.inc>
+#include <mobius/io/file.h>
 #include <mobius/io/folder.h>
 #include <cstddef>
 #include <stdexcept>
 #include <unordered_map>
 
-namespace mobius
-{
-namespace model
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+// Rationale:
+// 1. All item attributes are stored as serialized mobius::pod::data
+// 2. SQLite limits BLOB size to 1 GB (see https://www.sqlite.org/limits.html)
+// 3. If serialized data size is greater than ATTRIBUTE_FILE_THRESHOLD,
+//    this data is saved in an attribute file at item/data/attrs/<id>.pod and
+//    the attribute value is filled with ATTRIBUTE_FILE_ID.
+// 3. Conversely, when reading attributes, if an attribute has value equal to
+//    ATTRIBUTE_FILE_ID, then its value is read from attribute file.
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+
+namespace mobius::model
 {
 namespace
 {
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+// Constants
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+constexpr std::uint64_t ATTRIBUTE_FILE_THRESHOLD = 33554432; // 32 MiB
+const mobius::bytearray ATTRIBUTE_FILE_ID = {0xde, 0xea, 0xbe, 0xef, 0xc0, 0xc0, 0xa0, 'M', 'O', 'B', 'I', 'U', 'S', 'P', 'O', 'D'};
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Expand value mask, using item attributes
 //! \param value_mask value mask
@@ -111,11 +128,13 @@ public:
   mobius::pod::data get_attribute (const std::string&) const;
   void set_attribute (const std::string&, const mobius::pod::data&);
   void remove_attribute (const std::string&);
+  std::unordered_map <std::string, mobius::pod::data> get_attributes () const;
 
   item new_child (const std::string&, int);
   void remove ();
   void move (int, const item&);
   std::string get_data_path (const std::string&) const;
+  std::string create_data_path (const std::string&) const;
 
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   //! \brief Get uid
@@ -158,17 +177,6 @@ public:
     return case_.get_database ();
   }
 
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  //! \brief Get attributes
-  //! \return Attributes
-  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  std::unordered_map <std::string, mobius::pod::data>
-  get_attributes () const
-  {
-    _load_attributes ();
-    return attributes_;
-  }
-
 private:
   //! \brief Case object
   Case case_;
@@ -179,21 +187,19 @@ private:
   //! \brief Category
   mutable std::string category_;
 
-  //! \brief Attributes
-  mutable std::unordered_map <std::string, mobius::pod::data> attributes_;
-
   //! \brief Data loaded flag
   mutable bool data_loaded_ = false;
 
-  //! \brief Attributes loaded flag
-  mutable bool attributes_loaded_ = false;
-
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // Helper functions
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   void _load_data () const;
-  void _load_attributes () const;
   int _reserve_index (int) const;
+  void _save_attribute_file (const std::string&, const mobius::bytearray&);
+  mobius::bytearray _load_attribute_file (const std::string&) const;
+  void _remove_attribute_file (const std::string&);
+  mobius::io::file _get_attribute_file (const std::string&) const;
+  mobius::io::file _create_attribute_file (const std::string&) const;
 };
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -232,6 +238,20 @@ item::impl::get_data_path (const std::string& rpath) const
   return case_.get_path ("data/" + std::string (s_uid) + '/' + rpath);
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create data path
+//! \param rpath Relative path
+//! \return Fullpath
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::string
+item::impl::create_data_path (const std::string& rpath) const
+{
+  char s_uid[32];
+  sprintf (s_uid, "%04ld", get_uid ());
+
+  return case_.create_path ("data/" + std::string (s_uid) + '/' + rpath);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Check if item has attribute
 //! \param id Attribute ID
@@ -240,9 +260,23 @@ item::impl::get_data_path (const std::string& rpath) const
 bool
 item::impl::has_attribute (const std::string& id) const
 {
-  _load_attributes ();
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Build query
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  auto db = get_database ();
+  auto stmt = db.new_statement (
+                "SELECT 1 "
+                  "FROM attribute "
+                 "WHERE item_uid = ? "
+                   "AND id = ?");
 
-  return attributes_.find (id) != attributes_.end ();
+  stmt.bind (1, uid_);
+  stmt.bind (2, id);
+
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Return result
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  return bool (stmt.fetch_row ());
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -255,12 +289,31 @@ item::impl::get_attribute (const std::string& id) const
 {
   mobius::pod::data value;
 
-  _load_attributes ();
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Build query
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  auto db = get_database ();
+  auto stmt = db.new_statement (
+                "SELECT value "
+                  "FROM attribute "
+                 "WHERE item_uid = ? "
+                   "AND id = ?");
 
-  auto iter = attributes_.find (id);
-  
-  if (iter != attributes_.end ())
-    value = iter->second;
+  stmt.bind (1, uid_);
+  stmt.bind (2, id);
+
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Return result
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  if (stmt.fetch_row ())
+    {
+      auto bytes = stmt.get_column_bytearray (0);
+
+      if (bytes == ATTRIBUTE_FILE_ID)
+        bytes = _load_attribute_file (id);
+      
+      value = mobius::pod::unserialize (bytes);
+    }
 
   return value;	
 }
@@ -273,7 +326,28 @@ item::impl::get_attribute (const std::string& id) const
 void
 item::impl::set_attribute (const std::string& id, const mobius::pod::data& value)
 {
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Remove old attribute file, if any
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  auto f = _get_attribute_file (id);
+
+  if (f.exists ())
+    f.remove ();
+
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Serialize value and check if size is greater than threshold
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  mobius::bytearray bytes = mobius::pod::serialize (value);
+
+  if (bytes.size () > ATTRIBUTE_FILE_THRESHOLD)
+    {
+      _save_attribute_file (id, bytes);
+      bytes = ATTRIBUTE_FILE_ID;
+    }
+
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // Add to database
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   auto db = get_database ();
   mobius::database::statement stmt;
 
@@ -285,7 +359,7 @@ item::impl::set_attribute (const std::string& id, const mobius::pod::data& value
                 "WHERE item_uid = ? "
                   "AND id = ?");
 
-      stmt.bind (1, value);
+      stmt.bind (1, bytes);
       stmt.bind (2, uid_);
       stmt.bind (3, id);
     }
@@ -298,13 +372,10 @@ item::impl::set_attribute (const std::string& id, const mobius::pod::data& value
 
       stmt.bind (1, uid_);
       stmt.bind (2, id);
-      stmt.bind (3, value);
+      stmt.bind (3, bytes);
     }
 
   stmt.execute ();
-  
-  // Add to cache
-  attributes_[id] = value;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -314,9 +385,10 @@ item::impl::set_attribute (const std::string& id, const mobius::pod::data& value
 void
 item::impl::remove_attribute (const std::string& id)
 {
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // Remove from database
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   auto db = get_database ();
-
   auto stmt = db.new_statement (
                 "DELETE FROM attribute "
                       "WHERE item_uid = ? "
@@ -325,9 +397,48 @@ item::impl::remove_attribute (const std::string& id)
   stmt.bind (1, get_uid ());
   stmt.bind (2, id);
   stmt.execute ();
+  
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Remove attribute file, if any
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  _remove_attribute_file (id);
+}
 
-  // Remove from cache
-  attributes_.erase (id);
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Load attributes on demand
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::unordered_map <std::string, mobius::pod::data>
+item::impl::get_attributes () const
+{
+  std::unordered_map <std::string, mobius::pod::data> attributes;
+
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Build query
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  auto db = get_database ();
+  auto stmt = db.new_statement (
+                "SELECT id, value "
+                  "FROM attribute "
+                 "WHERE item_uid = ?");
+
+  stmt.bind (1, uid_);
+
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  // Fetch rows
+  // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  while (stmt.fetch_row ())
+    {
+      auto id = stmt.get_column_string (0);
+      auto bytes = stmt.get_column_bytearray (1);
+
+      if (bytes == ATTRIBUTE_FILE_ID)
+        bytes = _load_attribute_file (id);
+      
+      auto value = mobius::pod::unserialize (bytes);
+      attributes[id] = value;
+    }
+
+  return attributes;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -529,35 +640,6 @@ item::impl::_load_data () const
   data_loaded_ = true;
 }
 
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Load attributes on demand
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-void
-item::impl::_load_attributes () const
-{
-  if (attributes_loaded_)
-    return;
-
-  // load attributes
-  auto db = get_database ();
-
-  auto stmt = db.new_statement (
-                "SELECT id, value "
-                  "FROM attribute "
-                 "WHERE item_uid = ?");
-
-  stmt.bind (1, uid_);
-
-  while (stmt.fetch_row ())
-    {
-      auto id = stmt.get_column_string (0);
-      auto value = stmt.get_column_pod (1);
-      attributes_[id] = value;
-    }
-
-  attributes_loaded_ = true;
-}
-
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Reserve one slot for a child at position idx
 //! \param idx index, starting in 1 or -1 for last position
@@ -611,6 +693,71 @@ item::impl::_reserve_index (int idx) const
   return idx;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Get attribute file
+//! \param id Attribute ID
+//! \return File object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::io::file
+item::impl::_get_attribute_file (const std::string& id) const
+{
+  auto path = get_data_path ("attrs/" + id + ".pod");
+  return mobius::io::new_file_by_path (path);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create attribute file
+//! \param id Attribute ID
+//! \return File object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::io::file
+item::impl::_create_attribute_file (const std::string& id) const
+{
+  auto path = create_data_path ("attrs/" + id + ".pod");
+  return mobius::io::new_file_by_path (path);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Save attribute file
+//! \param id Attribute ID
+//! \param bytes Attribute data serialized
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+item::impl::_save_attribute_file (
+  const std::string& id,
+  const mobius::bytearray& bytes)
+{
+  auto f = _create_attribute_file (id);
+  auto writer = f.new_writer ();
+  writer.write (bytes);
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Load attribute file
+//! \param id Attribute ID
+//! \return Attribute data serialized
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::bytearray
+item::impl::_load_attribute_file (const std::string& id) const
+{
+  auto f = _get_attribute_file (id);
+  auto reader = f.new_reader ();
+  return reader.read (reader.get_size ());
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Remove attribute file, if any
+//! \param id Attribute ID
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+item::impl::_remove_attribute_file (const std::string& id)
+{
+  auto f = _get_attribute_file (id);
+
+  if (f.exists ())
+    f.remove ();
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Constructor
 //! \param c case object
@@ -712,7 +859,13 @@ item::set_attribute (const std::string& id, const mobius::pod::data& value)
   if (!impl_)
     throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
 
-  impl_->set_attribute (id, value);
+  auto old_value = impl_->get_attribute (id);
+
+  if (value != old_value)
+    {
+      impl_->set_attribute (id, value);
+      mobius::core::emit ("attribute-modified", *this, id, old_value, value);
+    }
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -725,7 +878,10 @@ item::remove_attribute (const std::string& id)
   if (!impl_)
     throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
 
+  auto old_value = impl_->get_attribute (id);
   impl_->remove_attribute (id);
+
+  mobius::core::emit ("attribute-removed", *this, id, old_value);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -910,6 +1066,7 @@ item::expand_masks ()
 
       if (!value_mask.empty ())
         {
+          auto old_value = get_attribute (attr.get_id ());
           std::string value = expand_value_mask (value_mask, *this);
           set_attribute (attr.get_id (), value);
         }
@@ -938,11 +1095,10 @@ item::get_data_path (const std::string& rpath) const
 std::string
 item::create_data_path (const std::string& rpath) const
 {
-  char s_uid[32];
-  sprintf (s_uid, "%04ld", get_uid ());
+  if (!impl_)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("item is null"));
 
-  auto c = get_case ();
-  return c.create_path ("data/" + std::string (s_uid) + '/' + rpath);
+  return impl_->create_data_path (rpath);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -954,7 +1110,7 @@ item::create_data_path (const std::string& rpath) const
 bool
 operator== (const item& a, const item& b)
 {
-  return a.get_uid () == b.get_uid ();
+  return a.get_uid () == b.get_uid () && a.get_case () == b.get_case ();
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -966,8 +1122,77 @@ operator== (const item& a, const item& b)
 bool
 operator!= (const item& a, const item& b)
 {
-  return a.get_uid () != b.get_uid ();
+  return a.get_uid () != b.get_uid () || a.get_case () != b.get_case ();
 }
 
-} // namespace model
-} // namespace mobius
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Operator <
+//! \param a item a
+//! \param b item b
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+operator< (const item& a, const item& b)
+{
+  return a.get_case () == b.get_case () && a.get_uid () < b.get_uid ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Operator <=
+//! \param a item a
+//! \param b item b
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+operator<= (const item& a, const item& b)
+{
+  return a.get_case () == b.get_case () && a.get_uid () <= b.get_uid ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Operator >
+//! \param a item a
+//! \param b item b
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+operator> (const item& a, const item& b)
+{
+  return a.get_case () == b.get_case () && a.get_uid () > b.get_uid ();
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Operator >=
+//! \param a item a
+//! \param b item b
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+operator>= (const item& a, const item& b)
+{
+  return a.get_case () == b.get_case () && a.get_uid () >= b.get_uid ();
+}
+
+} // namespace mobius::model
+
+namespace std
+{
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+// std::hash specialization for mobius::model::item
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+size_t
+std::hash <mobius::model::item>::operator () (const mobius::model::item& item) const
+{
+  constexpr std::uint64_t MUL = 0x9e3779b97f4a7c15ull; // 2^64/phi
+  auto item_uid = item.get_uid ();
+  auto case_uid = item.get_case ().get_uid ();
+
+  // Combine item UID and case UID
+  std::uint64_t seed = case_uid * MUL;
+  seed ^= (item_uid >> 32) * MUL + (seed << 6) + (seed >> 2);
+  seed ^= (item_uid & 0xffffffff) * MUL + (seed << 6) + (seed >> 2);
+
+  return seed;
+}
+
+} // namespace std
