@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -846,6 +846,56 @@ tp_f_get_ipv4_le (decoder_data_decoder_o *self, PyObject *)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>get_ipv6</i> method implementation
+//! \param self object
+//! \param args argument list
+//! \return IPv6 string
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_ipv6 (decoder_data_decoder_o *self, PyObject *)
+{
+  // execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pystring_from_std_string (self->obj->get_ipv6 ());
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  // return value
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>get_ipv4_mapped_ipv6</i> method implementation
+//! \param self object
+//! \param args argument list
+//! \return IPv6 string
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_ipv4_mapped_ipv6 (decoder_data_decoder_o *self, PyObject *)
+{
+  // execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pystring_from_std_string (self->obj->get_ipv4_mapped_ipv6 ());
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  // return value
+  return ret;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Methods structure
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -1025,6 +1075,18 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "get IPv4 (little endian) as string"
   },
+  {
+    (char *) "get_ipv6",
+    (PyCFunction) tp_f_get_ipv6,
+    METH_VARARGS,
+    "get IPv6 as string"
+  },
+  {
+    (char *) "get_ipv4_mapped_ipv6",
+    (PyCFunction) tp_f_get_ipv4_mapped_ipv6,
+    METH_VARARGS,
+    "get IPv6 as string"
+  },
   {NULL, NULL, 0, NULL} // sentinel
 };
 
@@ -1079,7 +1141,7 @@ static void
 tp_dealloc (decoder_data_decoder_o *self)
 {
   delete self->obj;
-  self->ob_type->tp_free ((PyObject*) self);
+  Py_TYPE (self)->tp_free ((PyObject*) self);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -1134,4 +1196,5 @@ PyTypeObject decoder_data_decoder_t =
   0,                                       		// tp_weaklist
   0,                                       		// tp_del
   0,                                       		// tp_version_tag
+  0,                                       		// tp_finalize
 };
