@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -368,6 +368,105 @@ tp_f_move (model_item_o *self, PyObject *args)
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>expand_masks</i> method implementation
+//! \param self object
+//! \param args argument list
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_expand_masks (model_item_o *self, PyObject *)
+{
+  PyObject *ret = nullptr;
+
+  try
+    {
+      self->obj->expand_masks ();
+      ret = mobius::py::pynone ();
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>get_data_path</i> method implementation
+//! \param self object
+//! \param args argument list
+//! \return full path
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_get_data_path (model_item_o *self, PyObject *args)
+{
+  // parse input args
+  std::string arg_rpath;
+
+  try
+    {
+      arg_rpath = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pystring_from_std_string (self->obj->get_data_path (arg_rpath));
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  // return value
+  return ret;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief <i>create_data_path</i> method implementation
+//! \param self object
+//! \param args argument list
+//! \return full path
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyObject *
+tp_f_create_data_path (model_item_o *self, PyObject *args)
+{
+  // parse input args
+  std::string arg_rpath;
+
+  try
+    {
+      arg_rpath = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
+  PyObject *ret = nullptr;
+
+  try
+    {
+      ret = mobius::py::pystring_from_std_string (self->obj->create_data_path (arg_rpath));
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_runtime_error (e.what ());
+    }
+
+  // return value
+  return ret;
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief <i>has_attribute</i> method implementation
 //! \param self object
@@ -546,94 +645,20 @@ tp_f_get_attributes (model_item_o *self, PyObject *)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief <i>expand_masks</i> method implementation
-//! \param self object
-//! \param args argument list
+//! \brief <i>has_vfs</i> method implementation
+//! \param self Object
+//! \param args Argument list
+//! \return true/false
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
-tp_f_expand_masks (model_item_o *self, PyObject *)
+tp_f_has_vfs (model_item_o *self, PyObject *)
 {
-  PyObject *ret = nullptr;
-
-  try
-    {
-      self->obj->expand_masks ();
-      ret = mobius::py::pynone ();
-    }
-  catch (const std::exception& e)
-    {
-      mobius::py::set_runtime_error (e.what ());
-    }
-
-  return ret;
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief <i>get_data_path</i> method implementation
-//! \param self object
-//! \param args argument list
-//! \return full path
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static PyObject *
-tp_f_get_data_path (model_item_o *self, PyObject *args)
-{
-  // parse input args
-  std::string arg_rpath;
-
-  try
-    {
-      arg_rpath = mobius::py::get_arg_as_std_string (args, 0);
-    }
-  catch (const std::exception& e)
-    {
-      mobius::py::set_invalid_type_error (e.what ());
-      return nullptr;
-    }
-
   // execute C++ code
   PyObject *ret = nullptr;
 
   try
     {
-      ret = mobius::py::pystring_from_std_string (self->obj->get_data_path (arg_rpath));
-    }
-  catch (const std::exception& e)
-    {
-      mobius::py::set_runtime_error (e.what ());
-    }
-
-  // return value
-  return ret;
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief <i>create_data_path</i> method implementation
-//! \param self object
-//! \param args argument list
-//! \return full path
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static PyObject *
-tp_f_create_data_path (model_item_o *self, PyObject *args)
-{
-  // parse input args
-  std::string arg_rpath;
-
-  try
-    {
-      arg_rpath = mobius::py::get_arg_as_std_string (args, 0);
-    }
-  catch (const std::exception& e)
-    {
-      mobius::py::set_invalid_type_error (e.what ());
-      return nullptr;
-    }
-
-  // execute C++ code
-  PyObject *ret = nullptr;
-
-  try
-    {
-      ret = mobius::py::pystring_from_std_string (self->obj->create_data_path (arg_rpath));
+      ret = mobius::py::pybool_from_bool (self->obj->has_vfs ());
     }
   catch (const std::exception& e)
     {
@@ -2544,6 +2569,24 @@ static PyMethodDef tp_methods[] =
     METH_VARARGS,
     "Move item"
   },
+  {
+    (char *) "expand_masks",
+    (PyCFunction) tp_f_expand_masks,
+    METH_VARARGS,
+    "Expand value masks using item's attributes"
+  },
+  {
+    (char *) "get_data_path",
+    (PyCFunction) tp_f_get_data_path,
+    METH_VARARGS,
+    "get item data path"
+  },
+  {
+    (char *) "create_data_path",
+    (PyCFunction) tp_f_create_data_path,
+    METH_VARARGS,
+    "create item data path"
+  },
   {
     (char *) "has_attribute",
     (PyCFunction) tp_f_has_attribute,
@@ -2575,22 +2618,10 @@ static PyMethodDef tp_methods[] =
     "Get attributes"
   },
   {
-    (char *) "expand_masks",
-    (PyCFunction) tp_f_expand_masks,
+    (char *) "has_vfs",
+    (PyCFunction) tp_f_has_vfs,
     METH_VARARGS,
-    "Expand value masks using item's attributes"
-  },
-  {
-    (char *) "get_data_path",
-    (PyCFunction) tp_f_get_data_path,
-    METH_VARARGS,
-    "get item data path"
-  },
-  {
-    (char *) "create_data_path",
-    (PyCFunction) tp_f_create_data_path,
-    METH_VARARGS,
-    "create item data path"
+    "Check if item has VFS"
   },
   {
     (char *) "has_ant",
