@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -29,7 +29,6 @@
 #include "bookmarked_url.h"
 #include "call.h"
 #include "case.h"
-#include "case_manager.h"
 #include "chat_message.h"
 #include "cookie.h"
 #include "encryption_key.h"
@@ -44,8 +43,60 @@
 #include "trash_can_entry.h"
 #include "visited_url.h"
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+// Functions prototypes
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+PyObject* func_model_new_case (PyObject *, PyObject *);
+PyObject* func_model_open_case (PyObject *, PyObject *);
+PyObject* func_model_close_case (PyObject *, PyObject *);
+PyObject* func_model_get_cases (PyObject *, PyObject *);
+PyObject* func_model_get_case_count (PyObject *, PyObject *);
+
 namespace
 {
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Module methods
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+static PyMethodDef module_methods[] =
+{
+  {
+    "new_case",
+    func_model_new_case,
+    METH_VARARGS,
+    "Create new case"
+  },
+  {
+    "open_case",
+    func_model_open_case,
+    METH_VARARGS,
+    "Open case"
+  },
+  {
+    "close_case",
+    func_model_close_case,
+    METH_VARARGS,
+    "Close case"
+  },
+  {
+    "get_cases",
+    func_model_get_cases,
+    METH_VARARGS,
+    "Get open cases"
+  },
+  {
+    "get_case_count",
+    func_model_get_case_count,
+    METH_VARARGS,
+    "Get number of open cases"
+  },
+  {
+    NULL,
+    NULL,
+    0,
+    NULL
+  } // sentinel
+};
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Module definition structure
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -55,7 +106,7 @@ static PyModuleDef module_def =
   "mobius.model",
   "Mobius Forensic Toolkit mobius.model module",
   -1,
-  nullptr,
+  module_methods,
   nullptr,
   nullptr,
   nullptr,
@@ -80,7 +131,6 @@ new_model_module ()
   module.add_type ("bookmarked_url", &model_bookmarked_url_t);
   module.add_type ("call", &model_call_t);
   module.add_type ("case", &model_case_t);
-  module.add_type ("case_manager", &model_case_manager_t);
   module.add_type ("chat_message", &model_chat_message_t);
   module.add_type ("cookie", &model_cookie_t);
   module.add_type ("encryption_key", &model_encryption_key_t);
