@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -208,14 +208,11 @@ tp_getter_data (os_win_credential_o *self)
 
   try
     {
-      auto f = [](const auto& p){
-        return mobius::py::pytuple_from_cpp_pair (
-                p,
+      ret = mobius::py::pylist_from_cpp_pair_container (
+                self->obj->get_data (),
                 mobius::py::pystring_from_std_string,
-                mobius::py::pybytes_from_bytearray);
-      };
-
-      ret = mobius::py::pylist_from_cpp_container (self->obj->get_data (), f);
+                mobius::py::pybytes_from_bytearray
+            );
     }
   catch (const std::exception& e)
     {
@@ -333,7 +330,7 @@ static void
 tp_dealloc (os_win_credential_o *self)
 {
   delete self->obj;
-  self->ob_type->tp_free ((PyObject*) self);
+  Py_TYPE (self)->tp_free ((PyObject*) self);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -388,4 +385,5 @@ PyTypeObject os_win_credential_t =
   0,                                       		// tp_weaklist
   0,                                       		// tp_del
   0,                                       		// tp_version_tag
+  0,                                       		// tp_finalize
 };
