@@ -1,6 +1,6 @@
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Mobius Forensic Toolkit
-# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Eduardo Aguiar
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -20,7 +20,7 @@ import string
 import json
 
 try:
-   # Python 2.6-2.7 
+   # Python 2.6-2.7
    from HTMLParser import HTMLParser
 except ImportError:
    # Python 3
@@ -113,6 +113,7 @@ EMOJI_CHARS = {
   'vampire'              : '🧛',
   'victory'              : '✌',
   'wasntme'              : '🙄',
+  'wave'                 : '🌊',
   'wink'                 : '😉',
   'worry'                : '😟',
   'xd'                   : '😆',
@@ -400,7 +401,7 @@ ENTITIES = {
 def unescape (text):
   html = HTMLParser ()
   return html.unescape (text)
-  
+
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief Skype message parser
 # @author Eduardo Aguiar
@@ -413,7 +414,7 @@ class MessageParser (object):
   def __init__ (self, text):
     self.__elements = []
     self.__text = text
-    
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Parse message
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -442,6 +443,25 @@ class MessageParser (object):
 
     return self.__elements
 
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  # @brief Parse notice
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  def parse_notice (self):
+    l = json.loads (self.__text)
+
+    if len (l) > 0:
+      c = l[0]
+      text = ''
+
+      for at in c.get ('attachments', []):
+        text += at.get ('content', {}).get ('text')
+
+      if text:
+        element = {'type' : 'system', 'text' : text}
+        self.add_element (element)
+
+    return self.__elements
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Parse popcard
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -450,10 +470,10 @@ class MessageParser (object):
 
     if len (l) > 0:
       c = l[0]
-      text = c.get ('content', {}).get (u'text')
+      text = c.get ('content', {}).get ('text')
 
       if text:
-        element = {u'type' : u'system', u'text' : text}
+        element = {'type' : 'system', 'text' : text}
         self.add_element (element)
 
     return self.__elements
@@ -462,43 +482,43 @@ class MessageParser (object):
   # @brief Parse <text>
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __parse_text (self, e):
-    element = {u'type' : u'text', u'text' : unicode (e.text, 'utf-8')}
-    self.add_element (element)    
+    element = {'type' : 'text', 'text' : e.text}
+    self.add_element (element)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Parse <tag>
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __parse_tag_open (self, e):
-    tag = unicode (e.text, 'utf-8')
+    tag = e.text
 
     f = getattr (self, '_parse_%s' % tag, None)
     if f:
       f (e)
 
     else:
-      mobius.core.log ('DEV app.skype: unknown tag open <%s>' % tag.encode ('utf-8'))
+      mobius.core.logf ('DEV app.skype: unknown tag open <%s>' % tag)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Parse </tag>
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __parse_tag_close (self, e):
     element = None
-    tag = unicode (e.text, 'utf-8')
+    tag = e.text
 
-    if tag == u'b':
-      element = {u'type' : u'end/b'}
+    if tag == 'b':
+      element = {'type' : 'end/b'}
 
-    elif tag == u'i':
-      element = {u'type' : u'end/i'}
+    elif tag == 'i':
+      element = {'type' : 'end/i'}
 
-    elif tag == u's':
-      element = {u'type' : u'end/s'}
+    elif tag == 's':
+      element = {'type' : 'end/s'}
 
-    elif tag == u'quote':
-      element = {u'type' : u'end/quote'}
+    elif tag == 'quote':
+      element = {'type' : 'end/quote'}
 
     else:
-      mobius.core.log ('DEV app.skype: unknown tag close </%s>' % tag.encode ('utf-8'))
+      mobius.core.logf ('DEV app.skype: unknown tag close </%s>' % tag)
 
     if element:
       self.add_element (element)
@@ -507,16 +527,16 @@ class MessageParser (object):
   # @brief Parse <tag/>
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __parse_tag_empty (self, e):
-    tag = unicode (e.text, 'utf-8')
-    mobius.core.log ('DEV app.skype: unknown tag empty <%s/>' % tag.encode ('utf-8'))
+    tag = e.text
+    mobius.core.logf ('DEV app.skype: unknown tag empty <%s/>' % tag)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Parse entity
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __parse_entity (self, e):
     element = {
-       u'type' : u'text',
-       u'text' : unescape (u'&' + unicode (e.text, 'utf-8') + u';')
+       'type' : 'text',
+       'text' : unescape ('&' + e.text + ';')
     }
     self.add_element (element)
 
@@ -526,16 +546,16 @@ class MessageParser (object):
   # @return Value
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __get_tag_value (self, e):
-    tag = unicode (e.text, 'utf-8')
+    tag = e.text
     value = None
-    
-    while e.type != 0 and (e.type != 3 or unicode (e.text, 'utf-8') != tag):
+
+    while e.type != 0 and (e.type != 3 or e.text != tag):
       if e.type == 1:
-        value = unicode (e.text, 'utf-8')
+        value = e.text
       e = self.__parser.get ()
 
     return value
-  
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Add text element to list
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -548,11 +568,11 @@ class MessageParser (object):
       p_element = None
 
     # merge two consecutive 'text' or 'system'
-    if p_element and element[u'type'] == p_element[u'type'] == u'text':
-        p_element[u'text'] = p_element[u'text'] + element[u'text']
+    if p_element and element['type'] == p_element['type'] == 'text':
+        p_element['text'] = p_element['text'] + element['text']
 
-    elif p_element and element[u'type'] == p_element[u'type'] == u'system':
-        p_element[u'text'] = p_element[u'text'] + '. ' + element[u'text']
+    elif p_element and element['type'] == p_element['type'] == 'system':
+        p_element['text'] = p_element['text'] + '. ' + element['text']
 
     # otherwise, create new element
     else:
@@ -562,28 +582,28 @@ class MessageParser (object):
   # @brief Parse <b> node
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def _parse_b (self, e):
-    element = {u'type' : u'start/b'}
+    element = {'type' : 'start/b'}
     self.add_element (element)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Parse <i> node
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def _parse_i (self, e):
-    element = {u'type' : u'start/i'}
+    element = {'type' : 'start/i'}
     self.add_element (element)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Parse <s> node
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def _parse_s (self, e):
-    element = {u'type' : u'start/s'}
+    element = {'type' : 'start/s'}
     self.add_element (element)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Parse <a> node
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def _parse_a (self, e):
-    element = {u'type' : u'href', u'url' : unescape (e.attributes.get (u'href'))}
+    element = {'type' : 'href', 'url' : unescape (e.attributes.get ('href'))}
     self.add_element (element)
 
     value = self.__get_tag_value (e)  # ignore text and </a>
@@ -592,22 +612,22 @@ class MessageParser (object):
   # @brief Parse <ss></ss> tag
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def _parse_ss (self, e):
-    element = {u'type' : u'emoji'}
+    element = {'type' : 'emoji'}
 
     # get ss_type
-    ss_type = e.attributes.get (u'type')
-  
+    ss_type = e.attributes.get ('type')
+
     if ss_type in EMOJI_CHARS:
-      element[u'code'] = EMOJI_CHARS.get (ss_type)
+      element['code'] = EMOJI_CHARS.get (ss_type)
 
     elif ss_type not in UNKNOWN_EMOJIS:
-      mobius.core.log ('DEV app.skype: unknown emoji "%s"' % ss_type)
+      mobius.core.logf ('DEV app.skype: unknown emoji "%s"' % ss_type)
       UNKNOWN_EMOJIS.add (ss_type)
 
     # get text
     text = self.__get_tag_value (e)
     if text:
-      element[u'text'] = text
+      element['text'] = text
 
     self.add_element (element)
 
@@ -615,22 +635,22 @@ class MessageParser (object):
   # @brief Parse <flag></flag> tag
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def _parse_flag (self, e):
-    element = {u'type' : u'flag'}
+    element = {'type' : 'flag'}
 
     # get country
-    country = e.attributes.get (u'country')
-  
+    country = e.attributes.get ('country')
+
     if country in FLAG_CHARS:
-      element[u'code'] = FLAG_CHARS.get (country)
+      element['code'] = FLAG_CHARS.get (country)
 
     elif country not in UNKNOWN_FLAGS:
-      mobius.core.log ('DEV app.skype: unknown flag "%s"' % country)
+      mobius.core.logf ('DEV app.skype: unknown flag "%s"' % country)
       UNKNOWN_FLAGS.add (country)
 
     # get text
     text = self.__get_tag_value (e)
     if text:
-      element[u'text'] = text
+      element['text'] = text
 
     self.add_element (element)
 
@@ -642,9 +662,11 @@ class MessageParser (object):
 
     # root node = <partlist>
     if e.type != 2 or e.text != 'partlist':
-      mobius.core.log ('WRN app.skype: invalid <partlist> ' + text)
+      mobius.core.logf ('WRN app.skype: invalid <partlist> ' + text)
       return
 
+    partlist_type = e.attributes.get ('type')
+
     # get all <part> nodes
     e = self.__parser.get ()
 
@@ -652,20 +674,30 @@ class MessageParser (object):
 
       if e.type == 2 and e.text == 'part':	# <part>
         part_id = e.attributes.get ('identity')
-        part_name = u''
+        part_name = ''
 
       elif e.type == 2 and e.text == 'name':	# <name>
         part_name = self.__get_tag_value (e)
-        
+
       elif e.type == 3 and e.text == 'part':	# </part>
         parts.append ((part_id, part_name))
 
       e = self.__parser.get ()
 
     # create element
-    text = u'Participants: ' + u', '.join (account + u' (' + name + u')' if name else account for (account, name) in sorted (parts))
+    if partlist_type == 'ended':
+      text = 'Call ended.'
 
-    element = {u'type' : u'system', u'text' : text}
+    elif partlist_type == 'started':
+      text = 'Call started.'
+
+    else:
+      mobius.core.logf ('DEV unknown partlist type "%s"' % partlist_type)
+      text = ''
+
+    text += ' Participants: ' + ', '.join (account + ' (' + name + ')' if name else account for (account, name) in sorted (parts))
+
+    element = {'type' : 'system', 'text' : text}
     self.add_element (element)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -676,7 +708,7 @@ class MessageParser (object):
 
     # root node = <contacts>
     if e.type != 2 or e.text != 'contacts':
-      mobius.core.log ('WRN app.skype: invalid <contacts> ' + e.text)
+      mobius.core.logf ('WRN app.skype: invalid <contacts> ' + e.text)
       return
 
     # get all <c> nodes
@@ -688,7 +720,7 @@ class MessageParser (object):
         t = e.attributes.get ('t')
 
         if t != 's':
-          mobius.core.log ('DEV app.skype: new contact "t" value: ' + t)
+          mobius.core.logf ('DEV app.skype: new contact "t" value: ' + t)
 
         account_id = e.attributes.get ('s')
         account_name = e.attributes.get ('f')
@@ -697,9 +729,9 @@ class MessageParser (object):
       e = self.__parser.get ()
 
     # create element
-    text = u'Contacts: ' + u', '.join (account + u' (' + name + u')' if name else account for (account, name) in sorted (contacts))
+    text = 'Contacts: ' + ', '.join (account + ' (' + name + ')' if name else account for (account, name) in sorted (contacts))
 
-    element = {u'type' : u'system', u'text' : text}
+    element = {'type' : 'system', 'text' : text}
     self.add_element (element)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -709,7 +741,7 @@ class MessageParser (object):
     filelist = []
 
     e = self.__parser.get ()
-    
+
     # get all <file> nodes
     while e.type != 0 and (e.type != 3 or e.text != 'files'):
 
@@ -722,7 +754,7 @@ class MessageParser (object):
           name = self.__get_tag_value (e)
         else:
           name = None
-    
+
         filelist.append ((size, status, timestamp, name))
 
       e = self.__parser.get ()
@@ -732,13 +764,13 @@ class MessageParser (object):
 
     if len (filelist) == 1:
       size, status, timestamp, name = filelist[0]
-      text = u'File "%s" sent (size: %d bytes)' % (name, size)
+      text = 'File "%s" sent (size: %d bytes)' % (name, size)
 
     elif len (filelist) > 1:
-      text = u'Files %s sent' % ', '.join ('"%s"' % name for size, status, timestamp, name in filelist)
+      text = 'Files %s sent' % ', '.join ('"%s"' % name for size, status, timestamp, name in filelist)
 
     if text:
-      element = {u'type' : u'system', u'text' : text}
+      element = {'type' : 'system', 'text' : text}
       self.add_element (element)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -753,7 +785,7 @@ class MessageParser (object):
     if author_name:
       author += ' (' + author_name + ')'
 
-    element = {u'type' : u'start/quote', u'author' : author}
+    element = {'type' : 'start/quote', 'author' : author}
     if timestamp:
       element['timestamp'] = mobius.datetime.new_datetime_from_unix_timestamp (int (timestamp))
 
@@ -772,17 +804,17 @@ class MessageParser (object):
     msg = e.attributes.get ('alt')
     timestamp = None
     targets = []
-    
+
     # process until </sms>
     e = self.__parser.get ()
 
     while e.type != 0 and (e.type != 3 or e.text != 'sms'):
-      etext = unicode (e.text, 'utf-8')
+      etext = e.text
 
-      if e.type == 2 and etext == u'sendtimestamp':
+      if e.type == 2 and etext == 'sendtimestamp':
         sendtimestamp = self.__get_tag_value (e)
 
-      elif e.type == 2 and etext == u'target':
+      elif e.type == 2 and etext == 'target':
         target = self.__get_tag_value (e)
         targets.append (target)
 
@@ -791,12 +823,12 @@ class MessageParser (object):
     # create element
     if timestamp:
       timestamp = mobius.datetime.new_datetime_from_unix_timestamp (int (timestamp))
-      text = u'SMS message sent at %s to %s: %s' % (timestamp, ', '.join (targets), msg)
+      text = 'SMS message sent at %s to %s: %s' % (timestamp, ', '.join (targets), msg)
 
     else:
-      text = u'SMS message sent to %s: %s' % (', '.join (targets), msg)
+      text = 'SMS message sent to %s: %s' % (', '.join (targets), msg)
 
-    element = {u'type' : u'system', u'text' : text}
+    element = {'type' : 'system', 'text' : text}
     self.add_element (element)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -813,82 +845,128 @@ class MessageParser (object):
     filesize = None
     description = None
     title = None
+    targets = []
 
     # process until </URIObject>
     e = self.__parser.get ()
 
     while e.type != 0 and (e.type != 3 or e.text != 'URIObject'):
-      etext = unicode (e.text, 'utf-8')
+      etext = e.text
 
       if e.type == 2:		# <tag>
-        if etext == u'a':
+        if etext == 'a':
           pass
 
-        elif etext == u'Description':
+        elif etext == 'Description':
           description = self.__get_tag_value (e)
 
-        elif etext == u'Title':
+        elif etext == 'Title':
           title = self.__get_tag_value (e)
 
-        elif etext == u'FileSize':
+        elif etext == 'FileSize':
           value = self.__get_tag_value (e)
           if value:
             filesize = int (value)
 
-        elif etext == u'OriginalName':
+        elif etext == 'OriginalName':
           name = e.attributes.get ('v')
 
-        elif etext == u'target':
+        elif etext == 'target':
           target = self.__get_tag_value (e)
           targets.append (target)
 
-        elif etext == u'meta':
+        elif etext == 'meta':
           meta_type = e.attributes.get ('type')
           meta_name = e.attributes.get ('originalName')
 
         else:
-          mobius.core.log ('DEV app.skype: unhandled URIObject sub-tag <%s>: %s' % (e.text, e.attributes.value))
+          mobius.core.logf ('DEV app.skype: unhandled URIObject sub-tag <%s>: %s' % (e.text, e.attributes.value))
 
       elif e.type == 4:		# <tag/>
-        if etext == u'Description':
+        if etext == 'Description':
           pass
-  
-        elif etext == u'FileSize':
-          filesize = int (e.attributes.get ('v'))
 
-        elif etext == u'meta':
+        elif etext == 'FileSize':
+          v = e.attributes.get ('v')
+          if v:
+            filesize = int (v)
+
+        elif etext == 'meta':
           meta_type = e.attributes.get ('type')
           meta_name = e.attributes.get ('originalName')
 
-        elif etext == u'OriginalName':
+        elif etext == 'OriginalName':
           name = e.attributes.get ('v')
 
-        elif etext == u'Title':
+        elif etext == 'Title':
           pass
-  
+
         else:
-          mobius.core.log ('DEV app.skype: unhandled URIObject sub-tag <%s/>: %s' % (e.text, e.attributes.value))
+          mobius.core.logf ('DEV app.skype: unhandled URIObject sub-tag <%s/>: %s' % (e.text, e.attributes.value))
 
       e = self.__parser.get ()
 
     # create element
-    text = u'File "%s" shared' % (name or meta_name)
+    text = 'File "%s" shared' % (name or meta_name)
 
     if type != None:
-      text += u'\nType: %s' % type
+      text += '\nType: %s' % type
 
     if filesize != None:
-      text += u'\nSize: %d bytes' % filesize
+      text += '\nSize: %d bytes' % filesize
 
     if title:
-      text += u'\nTitle: %s' % title
+      text += '\nTitle: %s' % title
 
     if description:
-      text += u'\nDescription: %s' % description
+      text += '\nDescription: %s' % description
 
-    text += u'\nURL: %s' % uri
+    text += '\nURL: %s' % uri
 
-    element = {u'type' : u'system', u'text' : text}
+    element = {'type' : 'system', 'text' : text}
+    self.add_element (element)
+
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  # @brief Parse <addmember> tag
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  def _parse_addmember (self, e):
+    timestamp = None
+    initiator = None
+    target = None
+
+    # process until </addmember>
+    e = self.__parser.get ()
+
+    while e.type != 0 and (e.type != 3 or e.text != 'addmember'):
+      etext = e.text
+
+      if e.type == 2:		# <tag>
+
+        if etext == 'eventtime':
+          s = int (self.__get_tag_value (e)) // 1000
+          timestamp = mobius.datetime.new_datetime_from_unix_timestamp (s)
+
+        elif etext == 'initiator':
+          initiator = format_account_id (self.__get_tag_value (e))
+
+        elif etext == 'target':
+          target = format_account_id (self.__get_tag_value (e))
+
+      e = self.__parser.get ()
+
+    # create element
+    text = 'Member "%s"' % target
+
+    if initiator and initiator != target:
+      text += ' added to chat by user "%s"' % initiator
+
+    else:
+      text += ' entered chat'
+
+    if timestamp:
+      text += ' at %s' % timestamp
+
+    element = {'type' : 'system', 'text' : text}
     self.add_element (element)
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -903,30 +981,117 @@ class MessageParser (object):
     e = self.__parser.get ()
 
     while e.type != 0 and (e.type != 3 or e.text != 'deletemember'):
-      etext = unicode (e.text, 'utf-8')
-      
+      etext = e.text
+
       if e.type == 2:		# <tag>
 
-        if etext == u'eventtime':
-          value = self.__get_tag_value (e)
-          timestamp = mobius.datetime.new_datetime_from_unix_timestamp (int (value) // 1000)
+        if etext == 'eventtime':
+          s = int (self.__get_tag_value (e)) // 1000
+          timestamp = mobius.datetime.new_datetime_from_unix_timestamp (s)
 
-        elif etext == u'initiator':
+        elif etext == 'initiator':
           initiator = format_account_id (self.__get_tag_value (e))
 
-        elif etext == u'target':
+        elif etext == 'target':
           target = format_account_id (self.__get_tag_value (e))
 
       e = self.__parser.get ()
 
     # create element
-    text = u'Member "%s" deleted from chat' % target
-    
-    if initiator:
-      text += u' by user "%s"' % initiator
-      
-    if timestamp:
-      text += u' in %s' % timestamp
+    text = 'Member "%s"' % target
 
-    element = {u'type' : u'system', u'text' : text}
+    if initiator and initiator != target:
+      text += ' removed from chat by user "%s"' % initiator
+
+    else:
+      text += ' left chat'
+
+    if timestamp:
+      text += ' at %s' % timestamp
+
+    element = {'type' : 'system', 'text' : text}
+    self.add_element (element)
+
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  # @brief Parse <topicupdate> tag
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  def _parse_topicupdate (self, e):
+    timestamp = None
+    initiator = None
+    value = None
+
+    # process until </topicupdate>
+    e = self.__parser.get ()
+
+    while e.type != 0 and (e.type != 3 or e.text != 'topicupdate'):
+      etext = e.text
+
+      if e.type == 2:		# <tag>
+
+        if etext == 'eventtime':
+          s = int (self.__get_tag_value (e)) // 1000
+          timestamp = mobius.datetime.new_datetime_from_unix_timestamp (s)
+
+        elif etext == 'initiator':
+          initiator = format_account_id (self.__get_tag_value (e))
+
+        elif etext == 'value':
+          value = self.__get_tag_value (e)
+
+      e = self.__parser.get ()
+
+    # create element
+    text = 'Topic updated to "%s"' % value
+
+    if initiator:
+      text += '  by user "%s"' % initiator
+
+    if timestamp:
+      text += ' at %s' % timestamp
+
+    element = {'type' : 'system', 'text' : text}
+    self.add_element (element)
+
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  # @brief Parse <historydisclosedupdate> tag
+  # @see https://docs.microsoft.com/en-us/javascript/api/botbuilder/iconversationupdate?view=botbuilder-ts-3.0#botbuilder-iconversationupdate-historydisclosed
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  def _parse_historydisclosedupdate (self, e):
+    timestamp = None
+    initiator = None
+    value = None
+
+    # process until </historydisclosedupdate>
+    e = self.__parser.get ()
+
+    while e.type != 0 and (e.type != 3 or e.text != 'historydisclosedupdate'):
+      etext = e.text
+
+      if e.type == 2:		# <tag>
+
+        if etext == 'eventtime':
+          s = int (self.__get_tag_value (e)) // 1000
+          timestamp = mobius.datetime.new_datetime_from_unix_timestamp (s)
+
+        elif etext == 'initiator':
+          initiator = format_account_id (self.__get_tag_value (e))
+
+        elif etext == 'value':
+          value = self.__get_tag_value (e)
+
+      e = self.__parser.get ()
+
+    # create element
+    if value == "True":
+      text = 'History disclosed'
+    else:
+      text = 'History closed'
+
+    if initiator:
+      text += '  by user "%s"' % initiator
+
+    if timestamp:
+      text += ' at %s' % timestamp
+
+    element = {'type' : 'system', 'text' : text}
     self.add_element (element)
