@@ -1,6 +1,6 @@
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Mobius Forensic Toolkit
-# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
+# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Eduardo Aguiar
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -16,8 +16,8 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 from pymobius.registry import *
+import pymobius
 import struct
-import binascii
 import mobius
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -64,7 +64,7 @@ class SAM_F (object):
     self.revision,                      \
     dummy1,                             \
     last_logon_timestamp,               \
-    dummy2,                             \
+    last_logoff_timestamp,              \
     last_password_set_timestamp,        \
     password_expiration_timestamp,      \
     last_failed_logon_timestamp,        \
@@ -72,19 +72,22 @@ class SAM_F (object):
     self.gid,                           \
     self.flags,                         \
     self.country_code,                  \
-    dummy3,                             \
+    self.codepage,                      \
     self.failed_logon_count,            \
     self.logon_count,                   \
     self.dummy4 = struct.unpack ('<IIQQQQQIIIHHHHI', data[:72])
-    
+
     if password_expiration_timestamp == 0x7fffffffffffffff:
       password_expiration_timestamp = 0
 
     # format data
     self.last_logon_datetime = get_nt_datetime (last_logon_timestamp)
+    self.last_logoff_datetime = get_nt_datetime (last_logoff_timestamp)
     self.last_password_set_datetime = get_nt_datetime (last_password_set_timestamp)
     self.last_failed_logon_datetime = get_nt_datetime (last_failed_logon_timestamp)
     self.password_expiration_datetime = get_nt_datetime (password_expiration_timestamp)
+
+    # @see https://docs.microsoft.com/en-us/openspecs/windows_protocols/ms-samr/b10cfda1-f24f-441b-8f43-80cb93e786ec
     self.flag_account_enabled = not (self.flags & 0x0001)
     self.flag_home_dir_required = bool (self.flags & 0x0002)
     self.flag_password_required = not (self.flags & 0x0004)
@@ -100,10 +103,10 @@ class SAM_F (object):
     self.__is_valid = True
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  # @brief check if object is valid
+  # @brief Check if object is valid
   # @return true/false
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  def __nonzero__ (self):
+  def is_valid (self):
     return self.__is_valid
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -248,7 +251,7 @@ class SAM_V (object):
     self.profile_path = self.__get_text (data, profile_path_offset, profile_path_size)
     self.workstation = self.__get_text (data, workstation_offset, workstation_size)
     self.hours_allowed = self.__get_field (data, hours_allowed_offset, hours_allowed_size)
-    self.dummy17 = dump (self.__get_field (data, dummy17_offset, dummy17_size))
+    self.dummy17 = pymobius.dump (self.__get_field (data, dummy17_offset, dummy17_size))
 
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     # Hashes
@@ -261,6 +264,13 @@ class SAM_V (object):
     # account is valid
     self.__is_valid = True
 
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  # @brief Check if object is valid
+  # @return true/false
+  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+  def is_valid (self):
+    return self.__is_valid
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Get UTF-16 field as UTF-8 text
   # @param data, offset, size
@@ -268,7 +278,7 @@ class SAM_V (object):
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __get_text (self, data, offset, size):
     pos = 0xcc
-    return data[pos + offset:pos + offset + size].decode ('utf-16').encode ('utf-8')
+    return data[pos + offset:pos + offset + size].decode ('utf-16')
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief get data field
@@ -311,7 +321,7 @@ class TS_User_Parameters (object):
     self.properties = []
     pos = 100
 
-    for i in xrange (property_count):
+    for i in range (property_count):
       prop = TS_Property (data[pos:])
       self.properties.append (prop)
       pos += prop.size
@@ -322,7 +332,7 @@ class TS_User_Parameters (object):
   # @brief check if object is valid
   # @return true/false
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  def __nonzero__ (self):
+  def is_valid (self):
     return self.__is_valid
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -362,10 +372,10 @@ class TS_Property (object):
     name_length, value_length, self.type = struct.unpack ('HHH', data[:6])
     pos = 6
 
-    self.name = data[pos : pos + name_length].decode ('utf-16').encode ('utf-8')
+    self.name = data[pos : pos + name_length].decode ('utf-16')
     pos += name_length
-    
-    value = binascii.unhexlify (data[pos : pos + value_length])
+
+    value = mobius.encoder.hexstring (data[pos : pos + value_length])
     pos += value_length
 
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -380,9 +390,9 @@ class TS_Property (object):
 
     elif self.name == 'CtxMinEncryptionLevel':
       self.value = ord (value[0])
-      
+
     elif self.name == 'msNPAllowDialin':
-      self.value = value.decode ('utf-16').encode ('utf-8').rstrip ('\0 ')
+      self.value = value.decode ('utf-16').rstrip ('\0 ')
 
     else:
       self.value = value
@@ -392,12 +402,12 @@ class TS_Property (object):
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     self.size = pos
     self.__is_valid = True
-    
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Check if object is valid
   # @return true/false
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  def __nonzero__ (self):
+  def is_valid (self):
     return self.__is_valid
 
 
@@ -415,7 +425,7 @@ def get_sam_key (registry):
     syskey = registry.get_syskey ()
     f = data.data
     revision = struct.unpack ('<I', f[:4])[0]
-    
+
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     # Revision 0x10002 (up to Win8.1)
     # @see http://moyix.blogspot.com.br/2008/02/syskey-and-sam.html
@@ -423,13 +433,13 @@ def get_sam_key (registry):
     if revision == 0x10002:
       md5 = mobius.crypt.hash ("md5")
       md5.update (f[0x70:0x80])
-      md5.update ('!@#$%^&*()qwertyUIOPAzxcvbnmQQQQQQQQQQQQ)(*@&%\0')
+      md5.update (b'!@#$%^&*()qwertyUIOPAzxcvbnmQQQQQQQQQQQQ)(*@&%\0')
       md5.update (syskey)
-      md5.update ('0123456789012345678901234567890123456789\0')
-      
+      md5.update (b'0123456789012345678901234567890123456789\0')
+
       rc4 = mobius.crypt.cipher ("rc4", md5.get_digest ())
       sam_key = rc4.encrypt (f[0x80:0xa0])[:16]
-    
+
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     # Revision 0x10003 (Win10)
     # @see https://github.com/gentilkiwi/mimikatz/wiki (kuhl_m_lsadump_getSamKey)
@@ -437,12 +447,12 @@ def get_sam_key (registry):
     elif revision == 0x10003:
       struct_revision, struct_size, check_len, data_len, salt = struct.unpack ('<IIII16s', f[0x68:0x88])
       data = f[0x88:0x88 + data_len]            # followed by check
-      
+
       aes = mobius.crypt.cipher ("aes", syskey, "cbc", salt)
       sam_key = aes.decrypt (data)[:16]         # AES-128 (16 bytes key)
 
     else:
-      mobius.core.logf ('Domains\\Account\\F unknown revision: %08x' % revision)
+      mobius.core.logf ('DEV Domains\\Account\\F unknown revision: %08x' % revision)
 
   return sam_key
 
@@ -463,7 +473,7 @@ class user_account (object):
     f = SAM_F (f.data)
     v = SAM_V (v.data)
 
-    if not f or not v:
+    if not f.is_valid () or not v.is_valid ():
       self.__is_valid = False
       return
 
@@ -490,7 +500,7 @@ class user_account (object):
     self.flag_server_trust_account = f.flag_server_trust_account
     self.flag_password_expires = f.flag_password_expires
     self.flag_account_auto_locked = f.flag_account_auto_locked
-    
+
     # set data from V value
     self.v_data_revision = v.revision
     self.username = v.username
@@ -502,25 +512,25 @@ class user_account (object):
     self.logon_script = v.logon_script
     self.profile_path = v.profile_path
     self.workstation = v.workstation
-    
-    if v.ts_user_parm:
+
+    if v.ts_user_parm.is_valid ():
       self.ts_properties = v.ts_user_parm.properties
     else:
       self.ts_properties = []
-      
+
     # decrypt password hashes
     self.hashes = []
 
     if v.nt_hash:
       self.hashes += self.__decrypt_user_hash (v.nt_hash, "nt", True)
 
-    if v.lm_hash:      
+    if v.lm_hash:
       self.hashes += self.__decrypt_user_hash (v.lm_hash, "lm", True)
 
     if v.nt_hash_history:
       self.hashes += self.__decrypt_user_hash (v.nt_hash_history, "nt", False)
 
-    if v.lm_hash_history:      
+    if v.lm_hash_history:
       self.hashes += self.__decrypt_user_hash (v.lm_hash_history, "lm", False)
 
     self.__is_valid = True
@@ -529,7 +539,7 @@ class user_account (object):
   # @brief Check if object is valid
   # @return true/false
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  def __nonzero__ (self):
+  def is_valid (self):
     return self.__is_valid
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -540,13 +550,13 @@ class user_account (object):
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def __decrypt_user_hash (self, data, htype, is_current):
     pekid, revision = struct.unpack ('<HH', data[:4])
-    
+
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     # DES final key1 and key2 values
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     rid_data = struct.pack ('<I', self.rid)
     key1 = rid_data + rid_data[:3]
-    key2 = rid_data[3] + rid_data + rid_data[:2]
+    key2 = rid_data[3:4] + rid_data + rid_data[:2]
 
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     # Revision 0x01 - Up to Win8.1
@@ -554,24 +564,24 @@ class user_account (object):
     if revision == 1:
       if htype == "nt":
         if is_current:
-          ntlm_str = "NTPASSWORD\0"
+          ntlm_str = b"NTPASSWORD\0"
         else:
-          ntlm_str = "NTPASSWORDHISTORY\0"
+          ntlm_str = b"NTPASSWORDHISTORY\0"
 
       elif htype == "lm":
         if is_current:
-          ntlm_str = "LMPASSWORD\0"
+          ntlm_str = b"LMPASSWORD\0"
         else:
-          ntlm_str = "LMPASSWORDHISTORY\0"
+          ntlm_str = b"LMPASSWORDHISTORY\0"
 
       md5 = mobius.crypt.hash ("md5")
       md5.update (self.__sam_key)
       md5.update (rid_data)
       md5.update (ntlm_str)
-      
+
       rc4 = mobius.crypt.cipher ("rc4", md5.get_digest ())
       obfkey = rc4.decrypt (data[4:])
-      
+
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     # Revision 0x02 - Win10
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -586,10 +596,10 @@ class user_account (object):
         obfkey = aes.decrypt (hdata)
 
       else:
-        obfkey = ''
+        obfkey = b''
 
     else:
-      mobius.core.logf ('SAM_HASH unknown revision: %d' % revision)
+      mobius.core.logf ('DEV SAM_HASH unknown revision: %d' % revision)
 
     # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
     # Get hashes from obfkey
@@ -597,7 +607,7 @@ class user_account (object):
     NTLM_HASH_SIZE = 16
     hashes = []
 
-    for i in xrange (0, len (obfkey), NTLM_HASH_SIZE):
+    for i in range (0, len (obfkey), NTLM_HASH_SIZE):
       h = passwd_hash ()
       h.revision = revision
       h.pekid = pekid
@@ -618,7 +628,7 @@ class user_account (object):
 def get (registry):
   accounts = []
   sam_key = get_sam_key (registry)
-  
+
   for key in registry.get_key_by_mask ('HKLM\\SAM\\SAM\\Domains\\Account\\Users\\[0-9]*'):
     f = key.get_data_by_name ('F')
     v = key.get_data_by_name ('V')
